import java.awt.Color;
import java.awt.Point;
import java.util.LinkedList;

/**
 * @author Milan Suchomel
 */
public class Zona {

    private LinkedList<Point> prazdneBody = new LinkedList<>();
    private LinkedList<Point> projduteBody = new LinkedList<>();
    private LinkedList<Pixel> barevneBody = new LinkedList<>();

    /**
     * Přidá prázdný bod do listu prázdných bodů
     * @param x Souřadnice X
     * @param y Souřadnice Y
     */
    public void pridejPrazdnyBod(int x, int y) {
        if (!prazdneBody.contains(new Point(x, y))) {
            prazdneBody.add(new Point(x, y));
        }
    }

    /**
     * Přidá barevný bod do listu barevných bodů
     * @param x Souřadnice X
     * @param y Souřadnice Y
     */
    public void pridejBarevnyBod(int x, int y, Color barva) {
        if (!barevneBody.contains(new Pixel(x, y, barva))) {
            barevneBody.add(new Pixel(x, y, barva));
        }
    }
    
    /**
     * Přidá projdutý bod do listu projdutých bodů
     * @param x Souřadnice X
     * @param y Souřadnice Y
     */
    public void pridejProjdutyBod(int x, int y) {
        if (!jeBodProjduty(x, y)) {
            projduteBody.add(new Point(x, y));
        }
    }

    /**
     * Zjistí zda obsahuje prázdný bod se souřadnicemi [X,Y]
     * @param x Souřadnice X
     * @param y Souřadnice Y
     * @return True, když je bod obsažen
     */
    public boolean obsahujePrazdnyBod(int x, int y) {
        return prazdneBody.contains(new Point(x, y));
    }

    /**
     * Zjistí zda bod se souřadnicemi [X,Y] je projdutý
     * @param x Souřadnice X
     * @param y Souřadnice Y
     * @return True, když je projdut
     */
    public boolean jeBodProjduty(int x, int y) {
        return projduteBody.contains(new Point(x, y));
    }

    /**
     * Doplní ke všem prázdným pixelům barvu
     * @return List doplněných pixelů
     */
    public LinkedList<Pixel> vratDokreslenouZonu() {
        LinkedList<Pixel> list = new LinkedList<>();
        for (Point bod : prazdneBody) {
            Barva barva = new Barva();
            for (Pixel pixel : barevneBody) {
                barva.prictiBarvu(pixel.getBarva(), bod.distance(pixel.getBod()));
            }
            list.add(new Pixel(bod.x, bod.y, barva.dejBarvu()));
        }
        return list;
    }
}

// Třída pro uchování pixelu, obsahuje bod a barvu
class Pixel {
    private Point bod;
    private Color barva;

    public Pixel(int x, int y, Color barva) {
        this.bod = new Point(x, y);
        this.barva = barva;
    }

    public Point getBod() {
        return bod;
    }

    public Color getBarva() {
        return barva;
    }
}

// Třída pro uchování více barev a následně vrácení průměrné barvy ... Nenapadl mě lepší název :)
class Barva {
    static final double KOEFICIENT = 3; // Mocnina vzdálenosti pro klasický přechod je přibližná hodnota "1.237"

    private double R = 0;
    private double G = 0;
    private double B = 0;
    private double A = 0;
    private double hodnost = 0; // Hodnota pro vážený průměr

    /**
     * Udělá průměrnou barvu ze všech barev, které byli přidány
     * @return Vrátí průměrnou barvu
     */
    Color dejBarvu() {
        return new Color((int)Math.round(R / hodnost), (int) Math.round(G / hodnost), (int) Math.round(B / hodnost), (int) Math.round(A / hodnost));
    }

    /**
     * Přičte barvu s její hodností ke všem předešlým barvám
     * @param barva Barva pixelu
     * @param vzdalenost Vzdalenost prazdného pixelu od pixelu krajního
     */
    void prictiBarvu(Color barva, double vzdalenost) {
        double h = 1 / Math.pow(vzdalenost, KOEFICIENT); // Podle vzdálenosti zjjistí jak moc ovlivní výslednou barvu

        this.R += barva.getRed() * h;
        this.G += barva.getGreen() * h;
        this.B += barva.getBlue() * h;
        this.A += barva.getAlpha() * h;
        this.hodnost += h;
    }
}
