import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.WritableRaster;
import java.util.LinkedList;

/**
 * @author Milan Suchome
 */
public class kresliciPlatno extends javax.swing.JPanel {

    public static final int PRAZDNY_RGB = (new Color(0, 0, 0, 0)).getRGB(); // Hexadecimální vyjádření prázdné barvy
    
    BufferedImage obrPuvodni = null; // Původní obrázek
    BufferedImage obrZmeneny = null; // Doplněný obrázek

    BufferedImage zobrazenyObrazek = null; // Vykreslovaný obrázek

    LinkedList<Zona> zony = new LinkedList<>(); // Zóny prázdných pixelů

    boolean jeZobrazenyZmeneny = false; // Boolean zda se má zobrazovate původní nebo doplněný obrázek

    MainJFrame main; // Reference na hlavní JFrame

    public kresliciPlatno() {
        initComponents();
    }

    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);

        // Určí jestli se zobrazí původní nebo dopněný obrázek
        if (!jeZobrazenyZmeneny) {
            zobrazenyObrazek = obrPuvodni;
        } else {
            zobrazenyObrazek = obrZmeneny;
        }

        // Vykreslí obrázek
        if (zobrazenyObrazek != null) {
            g.drawImage(zobrazenyObrazek, 0, 0, this);
        }

    }

    /**
     * Nastaví referenci na hlavní JFrame
     * @param main Hlavni JFrame
     */
    public void setMain(MainJFrame main) {
        this.main = main;
    }

    /**
     * Nastaví původní obrázek
     * @param obrazek Nekompletní obrázek
     */
    public void setImage(BufferedImage obrazek) {
        this.obrPuvodni = obrazek;
        this.obrZmeneny = zkopirujObrazek(obrazek);
        
        zmenObr(false);
        zony.clear(); // Vyčistí všechny zóny
        doplnObrazek();
    }
    
    public BufferedImage getImage(){
        return obrZmeneny;
    }

    /**
     * Vytvoří kopii ze zadaného obrázku
     * @param obrazek Předloha pro kopii
     * @return Kopii obrázku
     */
    private BufferedImage zkopirujObrazek(BufferedImage obrazek) {
        ColorModel cm = obrazek.getColorModel();
        boolean isAlphaPremultiplied = cm.isAlphaPremultiplied();
        WritableRaster raster = obrazek.copyData(null);
        return new BufferedImage(cm, raster, isAlphaPremultiplied, null);
    }

    /**
     * Přepne mezi původním a doplněným obrázkem
     * 
     * true - pro zobrazení změněného obrázku
     * false - pro zobrazení původního obrázku
     * @param bool Určí jaký se obrázek zobrazí
     */
    public void zmenObr(boolean bool) {
        this.jeZobrazenyZmeneny = bool;
        repaint();
    }

    /**
     * Doplní prázdné plochy původního obrázku 
     * a zakreslí je do obrázku změněného.
     */
    private void doplnObrazek() {
        for (int x = 0; x < obrZmeneny.getWidth(); x++) {
            for (int y = 0; y < obrZmeneny.getHeight(); y++) {
                if (obrZmeneny.getRGB(x, y) == PRAZDNY_RGB) { // Zjistí zda daný pixel je prázdný
                    if (soucastZony(x, y) == null) { // Zjistí zda bod je součástí jiné zóny
                        Zona novaZona = new Zona(); // Vytvoří novou zónu
                        pridejBodyDoZony(novaZona, x, y); // Přidá všechny prázdné přilehlé body do zóny
                        zony.add(novaZona); // Přidá zónu do lisut zón
                    }
                }
            }
        }

        for (Zona z : zony) {
            for (Pixel p : z.vratDokreslenouZonu()) {
                obrZmeneny.setRGB(p.getBod().x, p.getBod().y, p.getBarva().getRGB()); // Zakreslí všechny body ze všech zón do obrázku
            }
        }
    }

    /**
     * Přidá bod a jeho sousedící body do zóny
     * @param z Zóna do, které se body mají přidat
     * @param x Souřadnice X
     * @param y Souřadnice Y
     */
    private void pridejBodyDoZony(Zona z, int x, int y) {
        z.pridejPrazdnyBod(x, y); // Přidá prázdný bod do listu prázdných bodů zóny
        z.pridejProjdutyBod(x, y); // Určí že bod už je projdutý, jinak by se to zacyklilo
        LinkedList<Point> pomPrazdnyBody = new LinkedList(); // list sousedních bodů tohoto bodu, které nejsou ještě projduty
        Point bod;
        // Přídá bod vlevo od aktuálního bodu
        if (x - 1 >= 0) { // Zjistí jestli je bod součástí obrázku
            bod = pridejBodDoZony(z, x - 1, y); // Přidá bod do zóny
            if (bod != null) { // Projde jestli bod je prázdný
                if (!z.jeBodProjduty(bod.x, bod.y)) { // Zjistí zda je bod již projdutý
                    pomPrazdnyBody.add(bod); // Přidá do listu neprojdutých bodů
                }              
            }

        }   
        // Přidá bod vpravo od aktuálního bodu
        if (x + 1 < obrZmeneny.getWidth()) {
            bod = pridejBodDoZony(z, x + 1, y);
            if (bod != null) {
                if (!z.jeBodProjduty(bod.x, bod.y)) {
                    pomPrazdnyBody.add(bod);
                }              
            }
        }    
        // Přidá bod nad aktuálním bodem
        if (y - 1 >= 0) {
            bod = pridejBodDoZony(z, x, y - 1);
            if (bod != null) {
                if (!z.jeBodProjduty(bod.x, bod.y)) {
                    pomPrazdnyBody.add(bod);
                }              
            }
        }      
        // Přidá bod pod aktuálním bodem
        if (y + 1 < obrZmeneny.getHeight()) {
            bod = pridejBodDoZony(z, x, y + 1);
            if (bod != null) {
                if (!z.jeBodProjduty(bod.x, bod.y)) {
                    pomPrazdnyBody.add(bod);
                }              
            }
        }
      
        for (Point p : pomPrazdnyBody) {
            pridejBodyDoZony(z, p.x, p.y); // Přidá neprojdutý bod a jeho okolní body do zóny
        }
    }

    
    /**
     * Přidá bod do zóny
     * @param z Zona do, které se bod přidává
     * @param x Souřadnice X bodu
     * @param y Souřadnice Y bodu
     * @return Jestli je zadáný bod prázdný, vrátí ho, jestli plný vratí null
     */
    private Point pridejBodDoZony(Zona z, int x, int y) {
        if (obrZmeneny.getRGB(x, y) == PRAZDNY_RGB) {
            z.pridejPrazdnyBod(x, y);
            return new Point(x, y);
        } else {
            z.pridejBarevnyBod(x, y, new Color(obrZmeneny.getRGB(x, y), true));
            return null;
        }
    }

    /**
     * Zjistí zónu do, které bod patří
     * @param x Souřadnice X
     * @param y Souřadnice Y
     * @return  Vrátí zńu, do které bod patří, jestli nepatří do žádné vrátí null
     */
    private Zona soucastZony(int x, int y) {
        for (Zona z : zony) {
            if (z.obsahujePrazdnyBod(x, y)) {
                return z;
            }
        }
        return null;
    }


    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
            public void mouseMoved(java.awt.event.MouseEvent evt) {
                formMouseMoved(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 300, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents

    private void formMouseMoved(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseMoved
        main.vypisInformacePozice(evt.getX(), evt.getY());
        try {
            main.vypisInformaceRGB(zobrazenyObrazek.getRGB(evt.getX(), evt.getY()));
        } catch (Exception e) {           
        }
    }//GEN-LAST:event_formMouseMoved


    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
}
